<?php
/* --------------------------------------------------------------------------------------------------------
   Image Resizer for GetSimple
   
   This is modified code of Adaptive Images.
   Adaptive Images by Matt Wilcox is licensed under a Creative Commons Attribution 3.0 Unported License.
   See https://github.com/MattWilcox/Adaptive-Images

  ----------------------------------------------------------------------------------------------------------- */
  
require_once('../../../gsconfig.php');
$isDebug = defined('GSDEBUG') && GSDEBUG;

$host = $_SERVER['SERVER_NAME'] . ($_SERVER['SERVER_PORT'] != '80' ? ':'.$_SERVER['SERVER_PORT'] : '');
$pathParts = pathinfo($_SERVER['PHP_SELF']);
$subDir = str_replace('plugins/ImgResizer/img', '', $pathParts['dirname']);
$port = ($p=$_SERVER['SERVER_PORT'])!='80'&&$p!='443'?':'.$p:'';
$siteHost = http_protocol()."://". $host . $port;

//change this if you changed data dir for GS
$pluginDirPart = 'other/ImgResizer/';
$dataDirPart = 'data/';
$datadir = substr(dirname(__FILE__), 0, strrpos(dirname(__FILE__), DIRECTORY_SEPARATOR.'plugins')) . '/'.$dataDirPart;
$cachedir = $datadir.$pluginDirPart . 'cache/'; // where to store the generated re-sized images, remember to change in plugin this too if you need

$settings = null;
if (file_exists($datadir.$pluginDirPart . 'imgresizer_settings.php')){ //check that file with settingsexists
   include ($datadir.$pluginDirPart . 'imgresizer_settings.php');
   
   if (isset($imgresizer_settings))
        $settings = $imgresizer_settings;
}

$watch_cache   = TRUE; // check that the adapted image isn't stale (ensures updated source images are re-cached)
$browser_cache = 60*60*24*7; // How long the BROWSER cache should last (seconds, minutes, hours, days. 7days by default)

$requested_mode = @$_GET['mode'];
$resolution_string = @$_GET['resolution']; //store temporarly
$requested_width = null;
$requested_height = null;

$requested_path =  @$_GET['img']; //requested path

//delete site host and dir and data from image url
if ( substr($requested_path, 0, strlen($siteHost.$subDir.$dataDirPart) ) == $siteHost.$subDir.$dataDirPart) {
    $requested_path = substr($requested_path, strlen($siteHost.$subDir.$dataDirPart));
}
else if (substr($requested_path, 0, strlen($subDir.$dataDirPart) ) == $subDir.$dataDirPart){ //if its root relative (ex. /data/uploads/image.jpg)
    $requested_path = substr($requested_path, strlen($subDir.$dataDirPart));
}
else{ //wrong path, reset to fail later
    $requested_path = '';
}
   
//re add data dir part and safety replace ../ relative path 
$requested_path = removerelativepath( $requested_path );

if (strcasecmp(substr($requested_path, 0, strlen($pluginDirPart) ) , $pluginDirPart) == 0)
    sendError("Cannot use cached image as source.");


$requested_file = basename($requested_path);
$requested_path = substr($requested_path, 0, - strlen($requested_file)); //delete file name from path

$source_file    = $datadir.$requested_path.$requested_file;


// does the $cachedir directory exist already?
if (!is_dir($cachedir)) { // no
  if (!mkdir($cachedir, 0755, true)) { // so make it
    if (!is_dir($cachedir)) { // check again to protect against race conditions
      // uh-oh, failed to make that directory
      sendError("Failed to create cache directory at: $cachedir");
    }
  }
}


if (!$settings){
    sendError("Settings not found, configure plugin!");
}

//check requested by
if ($requested_mode !== 'height' && $requested_mode !== 'width' && $requested_mode !== 'fill' && $requested_mode !== 'fit'){
    sendError('Invalid "mode".');
}

//cast to integers
if ($requested_mode == 'height'){
	//check that requested size exists
	if (!in_array ( $resolution_string , $settings[$requested_mode], true )){
		sendError('Requested height not definied or 0.');
	}
	$requested_height = (int)$resolution_string; //cast to int
}
else if($requested_mode == 'width'){
	//check that requested size exists
	if (!in_array ( $resolution_string , $settings[$requested_mode], true )){
		sendError('Requested width not definied or 0.');
	}
	$requested_width = (int)$resolution_string; //
}
else{	
	$expl = @explode('x', $resolution_string); //$requested_width is temp storage
	if (count($expl) != 2)
		sendError('Requested resolution wrong definied.');
	$requested_height = (int)$expl[1];
	$requested_width = (int)$expl[0];
	//check that requested size exists, after casting to integers its values
	if (!in_array ( $resolution_string , $settings[$requested_mode], true ) || !$requested_width || !$requested_height ){
		sendError('Requested resolution not definied or 0.');
	}
}

// check if the file exists at all
if (!$requested_file || !file_exists($source_file)) {

  if (!$isDebug){
      header("Status: 404 Not Found");
      echo '404, not found: '.$requested_path.$requested_file;
  }
  else{
      sendError('Image not found.');
  }
  exit();
}

/* check that PHP has the GD library available to use for image re-sizing */
if (!extension_loaded('gd')) { // it's not loaded
  if (!function_exists('dl') || !dl('gd.so')) { // and we can't load it either
    // no GD available, so deliver the image straight up
    sendError('GD not available.');
  }
}


/* whew might the cache file be? */
$cache_file = $cachedir.'by-'.$requested_mode.'/'.$resolution_string.'/'.$requested_path.$requested_file;


if (file_exists($cache_file)) { // it exists cached at that size
  if ($watch_cache) { // if cache watching is enabled, compare cache and source modified dates to ensure the cache isn't stale
    $cache_file = refreshCache($source_file, $cache_file);
  }
  sendImage($cache_file, $browser_cache);
}

/* It exists as a source file, and it doesn't exist cached - lets make one: */
$file = generateImage($source_file, $cache_file);
sendImage($file, $browser_cache);

//removes relative path from path
function removerelativepath($file) {
    return preg_replace ('/\.\.?[\\\\\/]/', '' , $file);
}

/* helper function: Send headers and returns an image. */
function sendImage($filename, $browser_cache) {
  $extension = strtolower(pathinfo($filename, PATHINFO_EXTENSION));
  if (in_array($extension, array('png', 'gif', 'jpeg'))) {
    header("Content-Type: image/".$extension);
  } else {
    header("Content-Type: image/jpeg");
  }

  header("Cache-Control: private, max-age=".$browser_cache);
  header('Expires: '.gmdate('D, d M Y H:i:s', time()+$browser_cache).' GMT');
  header('Content-Length: '.filesize($filename));
  readfile($filename);
  exit();
}

/* helper function: Create and send an image with an error message. */
function sendError($message) {
  global $requested_width,$requested_height, $requested_mode,$requested_path,$requested_file,$isDebug;
  
  if (!$isDebug){
    header("Status: 404 Not Found");
    echo '404, error';
    exit;
  }

  $im         = ImageCreateTrueColor(800, 180);
  $text_color = ImageColorAllocate($im, 255, 255, 255);

  ImageString($im, 2, 5, 5, $message, $text_color);
  ImageString($im, 3, 5, 35, 'PARAMETERS', $text_color );
  ImageString($im, 3, 5, 55, 'img: '.@$_GET['img'], $text_color );
  ImageString($im, 3, 5, 75, 'path: '.$requested_path, $text_color );
  ImageString($im, 3, 5, 95, 'filename: '.$requested_file, $text_color );
  
  ImageString($im, 3, 5, 115, 'requested width: '.$requested_width . ' requested height: '.$requested_height, $text_color );
  ImageString($im, 3, 5, 135, 'mode: '.$requested_mode, $text_color );
  

  header("Cache-Control: no-store");
  header('Expires: '.gmdate('D, d M Y H:i:s', time()-1000).' GMT');
  header('Content-Type: image/jpeg');
  ImageJpeg($im);
  ImageDestroy($im);
  exit();
}

/* sharpen images function */
function findSharp($intOrig, $intFinal) {
  $intFinal = $intFinal * (750.0 / $intOrig);
  $intA     = 52; //changed from 52
  $intB     = -0.27810650887573124;
  $intC     = .00047337278106508946;
  $intRes   = $intA + $intB * $intFinal + $intC * $intFinal * $intFinal;
  return max(round($intRes), 0);
}

/* refreshes the cached image if it's outdated */
function refreshCache($source_file, $cache_file) {
  if (file_exists($cache_file)) {
    // not modified
    if (filemtime($cache_file) >= filemtime($source_file)) {
      return $cache_file;
    }

    // modified, clear it
    unlink($cache_file);
  }
  return generateImage($source_file, $cache_file);
}

/* generates the given cache file for the given source file with the given resolution and by height or width */
function generateImage($source_file, $cache_file) {
  global $settings, $requested_mode, $requested_width, $requested_height;

  $extension = strtolower(pathinfo($source_file, PATHINFO_EXTENSION));

  // Check the image dimensions
  $dimensions   = GetImageSize($source_file);
  $width        = $dimensions[0];
  $height       = $dimensions[1];
  
  if (!$dimensions){
    sendError('File is not proper image.');
  }

  if ($requested_mode == 'width'){
	if ($width <= $requested_width)
		return $source_file;
	
	$ratio = $height/$width;
	$new_width  = $requested_width;
	$new_height = round($new_width * $ratio);
  } 
  else if ($requested_mode == 'height'){
	if ($height <= $requested_height)
		return $source_file;
	
	$ratio = $width/$height;
	$new_height = $requested_height;
	$new_width = round($new_height * $ratio);    
  } 
  else if ($requested_mode == 'fit'){
    // So then if the image is wider rather than taller, set the width and figure out the height
    if ( ($width / $height ) > ($requested_width / $requested_height)) {
        $new_width = $requested_width;
        $new_height = ($requested_width * $height) / $width;
    }
    // And if the image is taller rather than wider, then set the height and figure out the width
    else if (($width / $height) < ($requested_width / $requested_height)) {
        $new_width = ($requested_height * $width)/ $height;
        $new_height = $requested_height;
    }
    // And because it is entirely possible that the image could be the exact same size/aspect ratio of the desired area, so we have that covered as well
    else{
        $new_width = $requested_width;
        $new_height = $requested_height;
    }

    //generating thumbnail not needed, image is smaller than thumb
    if ($width <= $requested_width && $height <= $requested_height)
        return $source_file;
  } 
  else if ($requested_mode == 'fill'){	
	if ($requested_width / $width < $requested_height / $height){
		$new_width = ceil($requested_height / $height * $width);
		$new_height = $requested_height;
		if ($new_width - $requested_width > 0) //only if needed
			$cropCoords = array (ceil(($new_width - $requested_width) / 2), 0);
	}
	else{
		$new_height = ceil($requested_width / $width * $height);
		$new_width = $requested_width;
		if ($new_height - $requested_heigh > 0) //only if needed
			$cropCoords = array (0, ceil(($new_height - $requested_height) / 2));
	}

	
	//generating thumbnail not needed, image is smaller than new thumb
    if ($width <= $new_width || $height <= $new_height)
        return $source_file;
  }
  
  $dst = ImageCreateTrueColor($new_width, $new_height); // re-sized image

  switch ($extension) {
    case 'png':
      $src = @ImageCreateFromPng($source_file); // original image
    break;
    case 'gif':
      $src = @ImageCreateFromGif($source_file); // original image
    break;
    default:
      $src = @ImageCreateFromJpeg($source_file); // original image
      ImageInterlace($dst, true); // Enable interlancing (progressive JPG, smaller size file)
    break;
  }

	if($extension=='png'){
		imagealphablending($dst, false);
		imagesavealpha($dst,true);
		$transparent = imagecolorallocatealpha($dst, 255, 255, 255, 127);
		imagefilledrectangle($dst, 0, 0, $new_width, $new_height, $transparent);
	}
  
	ImageCopyResampled($dst, $src, 0, 0, 0, 0, $new_width, $new_height, $width, $height); // do the resize in memory

	if (isset($cropCoords)){
		$cropped = ImageCreateTrueColor($requested_width, $requested_height); // re-sized image
		if($extension=='png'){
			imagealphablending($cropped, false);
			imagesavealpha($cropped,true);
			$transparent = imagecolorallocatealpha($cropped, 255, 255, 255, 127);
			imagefilledrectangle($cropped, 0, 0, $requested_width, $requested_height, $transparent);
		}
		ImageCopy($cropped, $dst, 0, 0, $cropCoords[0], $cropCoords[1], $new_width, $new_height);
        ImageDestroy($dst);
		$dst = $cropped;
	}


	ImageDestroy($src);

  // sharpen the image?
  //sharpening with transparent png gives some black pixels
  // NOTE: requires PHP compiled with the bundled version of GD (see http://php.net/manual/en/function.imageconvolution.php)
  if($settings['sharpen'] == '1' && function_exists('imageconvolution')) {
    $intSharpness = findSharp($width, $new_width);
    $arrMatrix = array(
      array(-1, -2, -1),
      array(-2, $intSharpness + 12, -2),
      array(-1, -2, -1)
    );
    imageconvolution($dst, $arrMatrix, $intSharpness, 0);
  }

  $cache_dir = dirname($cache_file);

  // does the directory exist already?
  if (!is_dir($cache_dir)) { 
    if (!mkdir($cache_dir, 0755, true)) {
      // check again if it really doesn't exist to protect against race conditions
      if (!is_dir($cache_dir)) {
        // uh-oh, failed to make that directory
        ImageDestroy($dst);
		
        sendError("Failed to create cache directory: $cache_dir");
      }
    }
  }

  if (!is_writable($cache_dir)) {
    sendError("The cache directory is not writable: $cache_dir");
  }

  // save the new file in the appropriate path, and send a version to the browser
  switch ($extension) {
    case 'png':
      $gotSaved = ImagePng($dst, $cache_file);
    break;
    case 'gif':
      $gotSaved = ImageGif($dst, $cache_file);
    break;
    default:
      $gotSaved = ImageJpeg($dst, $cache_file, $settings['quality']);
    break;
  }
  ImageDestroy($dst);

  if (!$gotSaved && !file_exists($cache_file)) {
    sendError("Failed to create image: $cache_file");
  }

  return $cache_file;
}



function http_protocol() {
	if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) {
	  return 'https';
	} else {
		return 'http';
	}
}
